/*
 * (c) Sinelabore Software Tools GmbH, 2008 - 2022
 *
 * All rights reserved. Reproduction, modification,
 * use or disclosure to third parties without express
 * authority is forbidden.
 */

#include <msp430.h> 
#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <stdio.h>
#include "pulse_len.h"
#include "dcf77_irq/dcf77_irq_ext.h"
#include "dcf77_irq/dcf77_irq.h"
#include "dcf77_decoder/decoder_ext.h"
#include "own_decoder_event.h"
#include "dcf77_decoder/decoder.h"


DECODER_INSTANCEDATA_T decoderSM=DECODER_INSTANCEDATA_INIT;
DCF77_IRQ_INSTANCEDATA_T irqSM = DCF77_IRQ_INSTANCEDATA_INIT;

static const char welcomeMsg[]="\r\nDCF77 receiver to demonstrate Sinelabore features\r\n";

// Send one character in busy waiting mode
void sendChar(char character){
    while (!(IFG2 & UTXIFG0));                // USART0 TX buffer ready?
    TXBUF0 = character;                          // RXBUF0 to TXBUF0
}

void sendString(char* str, uint8_t len){
    uint8_t n;
    for(n=0; n<len; n++){
        sendChar(*(str+n));
    }
}

extern uint8_t setClock(uint8_t minute, uint8_t hour,
                        uint8_t day, uint8_t dayOfWeek,
                        uint8_t month, uint8_t year){

    volatile uint8_t tempVar;

    _DINT();

    sendChar('\r');
    sendChar('\n');
    sendChar(((hour & 0x30) >> 4) + '0');
    sendChar((hour & 0x0f) + '0');
    sendChar(':');
    sendChar(((minute & 0x70) >> 4) + '0');
    sendChar((minute & 0x0f) + '0');
    sendChar(' ');
    sendChar(dayOfWeek+'0');
    sendChar(' ');
    sendChar(((day & 0x30) >> 4) + '0');
    sendChar((day & 0x0f) + '0');
    sendChar('/');
    sendChar(((month & 0x10) >> 4) + '0');
    sendChar((month & 0x0f) + '0');
    sendChar('/');
    sendChar(((year & 0xf0) >> 4) + '0');
    sendChar((year & 0x0f) + '0');
    sendChar('\r');
    sendChar('\n');
    sendChar('S');
    _EINT();

    return 0;
}


// From slac013o\MSP430F11x2_MSP430F12x_MSP430F12x2_Code_Examples
// File: fet120_uart05_09600.c
void initSerialInterface(){
    P3SEL |= 0x30;                            // P3.4,5 = USART0 TXD/RXD
    ME2 |= UTXE0 + URXE0;                     // Enable USART0 TXD/RXD
    UCTL0 |= CHAR;                            // 8-bit character
    UTCTL0 |= SSEL0;                          // UCLK = ACLK
    UBR00 = 0x03;                             // 32k/9600 - 3.41
    UBR10 = 0x00;                             //
    UMCTL0 = 0x4A;                            // Modulation
    UCTL0 &= ~SWRST;                          // Initialize USART state machine
    //IE2 |= URXIE0;                            // Enabled USART0 RX interrupt
}

void main(void)
{

    OWN_DECODER_EVENT_T decoderEvt;

    WDTCTL = WDTPW + WDTHOLD;             // Stop WDT
    TACTL = TASSEL0 + TACLR;              // ACLK, clear TAR
    CCTL0 = CCIE;                         // CCR0 interrupt enabled
    CCR0 = 125;
    TACTL |= MC0;

    P1DIR = 0x0;                          // by default all P1 as inputs
    P1DIR |= 0x0e;                         // P1.0 = dcf77 in /P1.1 LED / P1.2 and P1.3 debug outputs
    P3DIR = 0x08;                         // P3.3 debug output


    initSerialInterface();
    sendString(&welcomeMsg, strlen(welcomeMsg));

    _EINT();                              // Enable interrupts



    while(1) {
      _BIS_SR(LPM3_bits + GIE);                 // Enter LPM3
      __no_operation();

      P3OUT |= 0x8;

      decoderEvt.msg = irqSM.msg;
      decoderEvt.len = irqSM.len;

      // print debug information over serial port
      if(irqSM.msg == evMinStart){
          sendChar('M');
      }else if(irqSM.msg==evTick && irqSM.len==SHORT){
          sendChar('s');
      }else if(irqSM.msg==evTick && irqSM.len==LONG){
          sendChar('l');
      }
      decoder(&decoderSM, &decoderEvt);

      P3OUT &= ~(0x8);
    }

}


#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A0(void){

    // set debug output high if pin of dcf77 signal is high
    if((P1IN & 0x01) == 1U) P1OUT |= 0x04;
    else P1OUT &= ~(0x04);                  // clr debug pin

    P1OUT |= 0x08;                          // set debug len of irq

    dcf77_irq(&irqSM);
    if(irqSM.msg==evTick || irqSM.msg==evMinStart){
        // wake up main CPU, got valid pulse
        __bic_SR_register_on_exit(LPM3_bits);
    }

    P1OUT &= ~(0x08);                       // clr debug len of irq
}
